from wakeonlan import send_magic_packet
import requests
import json
import warnings
from pprint import pprint

warnings.simplefilter( "ignore" )

class API:

	def __init__( self , options={} ):
		self.options = options

	def pairing_stage_1( self , ip_address ):
		headers = {
			"Content-Type": "application/json" ,
		}
		data = {
			"_url": "/pairing/start" ,
			"DEVICE_ID": "pyvizio" ,
			"DEVICE_NAME": "Python Vizio" ,
		}
		url = f"https://{ip_address}:7345/pairing/start"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {
		# 	'ITEM': {'CHALLENGE_TYPE': 1, 'PAIRING_REQ_TOKEN': 802927 } ,
		# 	'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}
		# }
		result = json.loads( response.text )
		#pprint( result )
		return result["ITEM"]["PAIRING_REQ_TOKEN"]

	def pairing_stage_2( self , ip_address , pairing_request_token , code_displayed_on_tv ):
		headers = {
			"Content-Type": "application/json" ,
		}
		data = {
			"_url": "/pairing/pair" ,
			"DEVICE_ID": "pyvizio" ,
			"DEVICE_NAME": "Python Vizio" ,
			"CHALLENGE_TYPE": 1 ,
			"PAIRING_REQ_TOKEN": pairing_request_token ,
			"RESPONSE_VALUE": str( code_displayed_on_tv )
		}
		url = f"https://{ip_address}:7345/pairing/pair"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {
		# 	'ITEM': {'AUTH_TOKEN': 'Zhehzvszfq' } ,
		# 	'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}
		# }
		result = json.loads( response.text )
		#pprint( result )
		return result["ITEM"]["AUTH_TOKEN"]

	def get_power_state( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/state/device/power_mode"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [2308455925, 729988045],
		#  'ITEMS': [{'CNAME': 'volume',
		#             'ENABLED': 'FALSE',
		#             'HASHVAL': 1731828541,
		#             'NAME': 'Volume',
		#             'TYPE': 'T_VALUE_V1',
		#             'VALUE': 10}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/dynamic/tv_settings/audio/volume'}

		result = json.loads( response.text )
		#pprint( result )
		return result

	def power_on_silent( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 11 ,
				"CODE": 1 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}, 'URI': '/key_command/'}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def power_on( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 11 ,
				"CODE": 1 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}, 'URI': '/key_command/'}
		result = json.loads( response.text )
		send_magic_packet( self.options['mac_address'] )
		#pprint( result )
		return result

	def power_off( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 11 ,
				"CODE": 0 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}, 'URI': '/key_command/'}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_volume( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/audio/volume"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [2308455925, 729988045],
		#  'ITEMS': [{'CNAME': 'volume',
		#             'ENABLED': 'FALSE',
		#             'HASHVAL': 1731828541,
		#             'NAME': 'Volume',
		#             'TYPE': 'T_VALUE_V1',
		#             'VALUE': 10}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/dynamic/tv_settings/audio/volume'}

		result = json.loads( response.text )
		#pprint( result )
		return result["ITEMS"][0]["VALUE"]

	def volume_down( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 5 ,
				"CODE": 0 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def volume_up( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 5 ,
				"CODE": 1 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def mute_off( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 5 ,
				"CODE": 2 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def mute_on( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 5 ,
				"CODE": 3 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def mute_toggle( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 5 ,
				"CODE": 4 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_current_input( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/devices/current_input"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [1209501159, 1519436522],
		#  'ITEMS': [{'CNAME': 'current_input',
		#             'ENABLED': 'FALSE',
		#             'HASHVAL': 2690294191,
		#             'HIDDEN': 'TRUE',
		#             'NAME': 'Current Input',
		#             'TYPE': 'T_STRING_V1',
		#             'VALUE': 'hdmi2'}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/dynamic/tv_settings/devices/current_input'}
		result = json.loads( response.text )
		#pprint( result )
		return { "name": result["ITEMS"][0]["VALUE"] , "hash_value": result["ITEMS"][0]["HASHVAL"] }

	def get_available_inputs( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/devices/name_input"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()
		result = json.loads( response.text )
		# pprint( result )
		inputs = [ { "name": x["NAME"] , "hash_value": x["HASHVAL"] } for x in result["ITEMS"] ]
		return inputs

	def set_input( self , input_name ):

		# For this one, you first need the hash value of the current input id
		current_input = self.get_current_input()

		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/menu_native/dynamic/tv_settings/devices/current_input" ,
			"item_name": "CURRENT_INPUT" ,
			"VALUE": input_name ,
			"HASHVAL": current_input["hash_value"] ,
			"REQUEST": "MODIFY"
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/devices/current_input"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return

		result = json.loads( response.text )
		#pprint( result )
		return result

	def cycle_input( self ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/key_command/" ,
			"KEYLIST": [{
				"CODESET": 7 ,
				"CODE": 1 ,
				"ACTION": "KEYPRESS"
			}]
		}
		url = f"https://{self.options['ip']}:7345/key_command/"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}, "URI": "/key_command/"}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_audio_settings( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/audio"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_audio_setting( self , audio_setting ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/audio/{audio_setting}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [3552915159, 3436748430],
		#  'ITEMS': [{'CNAME': 'tv_speakers',
		#             'HASHVAL': 4126758801,
		#             'INDEX': 0,
		#             'NAME': 'Speakers',
		#             'TYPE': 'T_LIST_V1',
		#             'VALUE': 'Auto'}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/dynamic/tv_settings/audio/tv_speakers'}
		result = json.loads( response.text )
		#pprint( result )
		return { "setting": result["ITEMS"][0]["VALUE"] , "hash_value": result["ITEMS"][0]["HASHVAL"] }

	def get_all_audio_settings_options( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/static/tv_settings/audio"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_audio_settings_options( self , audio_setting_option ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/static/tv_settings/audio/{audio_setting_option}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHVAL': 1234567890,
		#  'ITEMS': [{'CNAME': 'tv_speakers',
		#             'ELEMENTS': ['Auto', 'On', 'Off'],
		#             'GROUP': 'G_AUDIO',
		#             'NAME': 'Speakers',
		#             'TYPE': 'T_LIST_V1'}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/static/tv_settings/audio/tv_speakers'}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def set_audio_setting( self , audio_setting , audio_setting_option ):

		# For this one, you first need the hash value of the current audio setting
		current_settings = self.get_audio_setting( audio_setting )

		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": f"/menu_native/dynamic/tv_settings/audio/{audio_setting}" ,
			"item_name": "SETTINGS" ,
			"VALUE": audio_setting_option ,
			"HASHVAL": current_settings["hash_value"] ,
			"REQUEST": "MODIFY"
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/audio/{audio_setting}"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [3552915159, 3436748430],
		#  'ITEMS': [{'HASHVAL': 2210383572, 'NAME': 'Mute'}],
		#  'PARAMETERS': {'HASHVAL': 1580137992, 'REQUEST': 'MODIFY', 'VALUE': 'Off'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_settings_types( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		result = json.loads( response.text )
		# pprint( result )
		return result

	def get_all_settings_for_type( self , setting_type ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/{setting_type}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_setting( self , setting_type , setting_name ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/{setting_type}/{setting_name}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [2880302639, 3367486168],
		#  'ITEMS': [{'CNAME': 'backlight',
		#             'HASHVAL': 1656752721,
		#             'NAME': 'Backlight',
		#             'TYPE': 'T_VALUE_V1',
		#             'VALUE': 100}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		#  'URI': '/menu_native/dynamic/tv_settings/picture/backlight'}
		result = json.loads( response.text )
		#pprint( result )
		return result

	def get_all_settings_options_for_type( self , settings_type ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/static/tv_settings/{settings_type}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		result = json.loads( response.text )
		#pprint( result )
		return result


	def get_settings_option( self , settings_type , setting_name ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/menu_native/static/tv_settings/{settings_type}/{setting_name}"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHVAL': 1234567890,
		#  'ITEMS': [{'CNAME': 'backlight',
		#             'GROUP': 'G_PICTURE',
		#             'INCREMENT': 1,
		#             'MAXIMUM': 100,
		#             'MINIMUM': 0,
		#             'NAME': 'Backlight',
		#             'TYPE': 'T_VALUE_V1'}],
		#  'PARAMETERS': {'FLAT': 'TRUE', 'HASHONLY': 'FALSE', 'HELPTEXT': 'FALSE'},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		result = json.loads( response.text )
		#pprint( result )
		return result

	def set_settings_option( self , settings_type , setting_name , settings_option ):

		# For this one, you first need the hash value of the current setting
		current_setting = self.get_setting( settings_type , setting_name )

		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": f"/menu_native/dynamic/tv_settings/{settings_type}/{setting_name}" ,
			"item_name": "SETTINGS" ,
			"VALUE": int( settings_option ) ,
			"HASHVAL": current_setting["ITEMS"][0]["HASHVAL"] ,
			"REQUEST": "MODIFY"
		}
		url = f"https://{self.options['ip']}:7345/menu_native/dynamic/tv_settings/{settings_type}/{setting_name}"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()

		# Should Return
		# {'HASHLIST': [791350083, 2564833227],
		#  'ITEMS': [{'HASHVAL': 2398756231, 'NAME': 'Backlight'}],
		#  'PARAMETERS': {'HASHVAL': 1656752721, 'REQUEST': 'MODIFY', 'VALUE': 90},
		#  'STATUS': {'DETAIL': 'Success', 'RESULT': 'SUCCESS'},
		result = json.loads( response.text )
		#pprint( result )
		return result

	# Look Here to Find APP_ID 's , Namespace Integers , and Messages
	# https://github.com/vkorn/pyvizio/blob/master/pyvizio/const.py
	def launch_app_config( self , app_id , name_space , message="None" ):
		headers = {
			"Content-Type": "application/json" ,
			"AUTH": self.options["access_token"]
		}
		data = {
			"_url": "/app/launch",
			"VALUE": {
				"APP_ID": str( app_id ) ,
				"NAME_SPACE": int( name_space ) ,
				"MESSAGE": message
			}
		}
		url = f"https://{self.options['ip']}:7345/app/launch"
		response = requests.put( url , headers=headers , data=json.dumps( data ) , verify=False )
		response.raise_for_status()
		#print( response.text )
		result = json.loads( response.text )
		return result

	def get_current_app( self ):
		headers = {
			'AUTH': self.options["access_token"]
		}
		url = f"https://{self.options['ip']}:7345/app/current"
		response = requests.get( url , headers=headers , verify=False )
		response.raise_for_status()
		result = json.loads( response.text )
		# Should Return
		# {"STATUS": {"RESULT": "SUCCESS", "DETAIL": "Success"}
		#pprint( result )
		return result


import inspect
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor

def _x_batch_process_exec_next_iter( options ):
	if len( options ) > 2:
		result = options[ 1 ]( *options[ 2 ] )
	else:
		result = options[ 1 ]()
	return result

def _x_batch_process( options ):
	batch_size = len( options[ "batch_list" ] )
	with ThreadPoolExecutor() as executor:
		# result_pool = list( tqdm( executor.map( batch_process_exec_next_iter , iter( options[ "batch_list" ] ) ) , total=batch_size ) )
		result_pool = list( executor.map( _x_batch_process_exec_next_iter , iter( options[ "batch_list" ] ) ) )
		return result_pool

# https://stackoverflow.com/q/15608987
def stack_reach( name ):
	for f in inspect.stack():
		if name in f[ 0 ].f_locals:
			return f[ 0 ].f_locals[ name ]
	return None

def get_parent_class_context():
	for f in inspect.stack():
		if "__name__" not in f[ 0 ].f_locals:
			continue
		print( f[ 0 ].f_locals )
	return False

class Settings:

	def __init__( self , options={} ):
		self.options = options
		try:
			self.this = get_parent_class_context()
			self.api = self.this.__dict__[ "API" ]
		except Exception as e:
			print( e )

	def test( self ):
		#print( self.api )
		for f in inspect.stack():
			if "__name__" not in f[ 0 ].f_locals:
				continue
			# print( f[ 0 ].f_locals )
			print( f[ 0 ].f_locals[ "viziocontroller" ].__dict__[ "API" ].get_setting )

	def get_picture_settings( self ):
		batch_list = [
			[ "picture_mode" , self.api.get_setting , [ self , "picture" , "picture_mode" ] ] ,
			[ "backlight" , self.api.get_setting , [ self , "picture" , "backlight" ] ] ,
			[ "brightness" , self.api.get_setting , [ self , "picture" , "brightness" ] ] ,
			[ "contrast" , self.api.get_setting , [ self , "picture" , "contrast" ] ] ,
			[ "color" , self.api.get_setting , [ self , "picture" , "color" ] ] ,
			[ "tint" , self.api.get_setting , [ self , "picture" , "tint" ] ] ,
			[ "sharpness" , self.api.get_setting , [ self , "picture" , "sharpness" ] ] ,
			[ "more_picture" , self.api.get_setting , [ self , "picture" , "more_picture" ] ]
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"picture_mode": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"backlight": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"brightness": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"contrast": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"color": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"tint": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"sharpness": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"more_picture": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_audio_settings( self ):
		batch_list = [
			[ "tv_speakers" , self.api.get_setting , [ self , "audio" , "tv_speakers" ] ] ,
			[ "volume_control_display" , self.api.get_setting , [ self , "audio" , "volume_control_display" ] ] ,
			[ "surround_sound" , self.api.get_setting , [ self , "audio" , "surround_sound" ] ] ,
			[ "volume_leveling" , self.api.get_setting , [ self , "audio" , "volume_leveling" ] ] ,
			[ "balance" , self.api.get_setting , [ self , "audio" , "balance" ] ] ,
			[ "lip_sync" , self.api.get_setting , [ self , "audio" , "lip_sync" ] ] ,
			[ "digital_audio_out" , self.api.get_setting , [ self , "audio" , "digital_audio_out" ] ] ,
			[ "analog_audio_out" , self.api.get_setting , [ self , "audio" , "analog_audio_out" ] ] ,
			[ "volume" , self.api.get_setting , [ self , "audio" , "volume" ] ] ,
			[ "mute" , self.api.get_setting , [ self , "audio" , "mute" ] ] ,
			[ "equalizer" , self.api.get_setting , [ self , "audio" , "equalizer" ] ] ,
			[ "delete_audio_mode" , self.api.get_setting , [ self , "audio" , "delete_audio_mode" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"tv_speakers": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"volume_control_display": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"surround_sound": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"volume_leveling": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"balance": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"lip_sync": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"digital_audio_out": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"analog_audio_out": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"volume": result[ 8 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"mute": result[ 9 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"equalizer": result[ 10 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"delete_audio_mode": result[ 11 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_network_settings( self ):
		batch_list = [
			[ "connection_type" , self.api.get_setting , [ self , "network" , "connection_type" ] ] ,
			[ "connection_status" , self.api.get_setting , [ self , "network" , "connection_status" ] ] ,
			[ "dhcp_mode" , self.api.get_setting , [ self , "network" , "dhcp_mode" ] ] ,
			[ "dhcp_mode" , self.api.get_setting , [ self , "network" , "dhcp_mode" ] ] ,
			[ "current_ssid_name" , self.api.get_setting , [ self , "network" , "current_ssid_name" ] ] ,
			[ "ip_address" , self.api.get_setting , [ self , "network" , "ip_address" ] ] ,
			[ "wireless_access_points" , self.api.get_setting , [ self , "network" , "wireless_access_points" ] ] ,
			[ "current_access_point" , self.api.get_setting , [ self , "network" , "current_access_point" ] ] ,
			[ "current_access_point" , self.api.get_setting , [ self , "network" , "current_access_point" ] ] ,
			[ "forget_wifi_network" , self.api.get_setting , [ self , "network" , "forget_wifi_network" ] ] ,
			[ "manual_setup" , self.api.get_setting , [ self , "network" , "manual_setup" ] ] ,
			[ "manual_setup_info" , self.api.get_setting , [ self , "network" , "manual_setup_info" ] ] ,
			[ "hidden_network" , self.api.get_setting , [ self , "network" , "hidden_network" ] ] ,
			[ "hidden_network_info" , self.api.get_setting , [ self , "network" , "hidden_network_info" ] ] ,
			[ "test_connection" , self.api.get_setting , [ self , "network" , "test_connection" ] ] ,
			[ "start_ap_search" , self.api.get_setting , [ self , "network" , "start_ap_search" ] ] ,
			[ "stop_ap_search" , self.api.get_setting , [ self , "network" , "stop_ap_search" ] ] ,
			[ "set_wifi_password" , self.api.get_setting , [ self , "network" , "set_wifi_password" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"connection_type": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"connection_status": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"dhcp_mode": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"dhcp_mode": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"current_ssid_name": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"ip_address": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"wireless_access_points": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"current_access_point": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"current_access_point": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"forget_wifi_network": result[ 8 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"manual_setup": result[ 9 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"manual_setup_info": result[ 10 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"hidden_network": result[ 11 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"hidden_network_info": result[ 12 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"test_connection": result[ 13 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"start_ap_search": result[ 14 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"stop_ap_search": result[ 15 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"set_wifi_password": result[ 16 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_channel_settings( self ):
		batch_list = [
			[ "auto_channel_scan" , self.api.get_setting , [ self , "channels" , "auto_channel_scan" ] ] ,
			[ "cancel_channel_scan" , self.api.get_setting , [ self , "channels" , "cancel_channel_scan" ] ] ,
			[ "analog" , self.api.get_setting , [ self , "channels" , "analog" ] ] ,
			[ "digital" , self.api.get_setting , [ self , "channels" , "digital" ] ] ,
			[ "percent_complete" , self.api.get_setting , [ self , "channels" , "percent_complete" ] ] ,
			[ "skip_channel" , self.api.get_setting , [ self , "channels" , "skip_channel" ] ] ,
			[ "analog_audio" , self.api.get_setting , [ self , "channels" , "analog_audio" ] ] ,
			[ "digital_language" , self.api.get_setting , [ self , "channels" , "digital_language" ] ] ,
			[ "parental_controls" , self.api.get_setting , [ self , "channels" , "parental_controls" ] ] ,
			[ "current_channel" , self.api.get_setting , [ self , "channels" , "current_channel" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"auto_channel_scan": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"cancel_channel_scan": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"analog": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"digital": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"percent_complete": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"skip_channel": result[ 5 ] ,
			"analog_audio": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"digital_language": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"parental_controls": result[ 8 ] ,
			"current_channel": result[ 9 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}


	def get_timer_settings( self ):
		batch_list = [
			[ "sleep_timer" , self.api.get_setting , [ self , "timers" , "sleep_timer" ] ] ,
			[ "auto_power_off_timer" , self.api.get_setting , [ self , "timers" , "auto_power_off_timer" ] ] ,
			[ "blank_screen" , self.api.get_setting , [ self , "timers" , "blank_screen" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"sleep_timer": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"auto_power_off_timer": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			# "auto_power_off_timer": result[ 1 ] ,
			"blank_screen": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_closed_caption_settings( self ):
		batch_list = [
			[ "closed_captions_enabled" , self.api.get_setting , [ self , "closed_captions" , "closed_captions_enabled" ] ] ,
			[ "analog_closed_captions" , self.api.get_setting , [ self , "closed_captions" , "analog_closed_captions" ] ] ,
			[ "digital_closed_captions" , self.api.get_setting , [ self , "closed_captions" , "digital_closed_captions" ] ] ,
			[ "digital_style" , self.api.get_setting , [ self , "closed_captions" , "digital_style" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"closed_captions_enabled": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"analog_closed_captions": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"digital_closed_captions": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"digital_style": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_device_settings( self ):
		batch_list = [
			[ "name_input" , self.api.get_setting , [ self , "devices" , "name_input" ] ] ,
			[ "hdmi1" , self.api.get_setting , [ self , "devices" , "hdmi1" ] ] ,
			[ "hdmi2" , self.api.get_setting , [ self , "devices" , "hdmi2" ] ] ,
			[ "comp" , self.api.get_setting , [ self , "devices" , "comp" ] ] ,
			[ "tuner" , self.api.get_setting , [ self , "devices" , "tuner" ] ] ,
			[ "current_input" , self.api.get_setting , [ self , "devices" , "current_input" ] ] ,
			[ "current_inputs" , self.api.get_setting , [ self , "devices" , "current_inputs" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"name_input": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"hdmi1": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"hdmi2": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"comp": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"tuner": result[ 4 ] ,
			"current_input": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"current_inputs": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_system_settings( self ):
		batch_list = [
			[ "service_check" , self.api.get_setting , [ self , "system" , "service_check" ] ] ,
			[ "system_information" , self.api.get_setting , [ self , "system" , "system_information" ] ] ,
			[ "menu_language" , self.api.get_setting , [ self , "system" , "menu_language" ] ] ,
			[ "local_time_settings" , self.api.get_setting , [ self , "system" , "local_time_settings" ] ] ,
			[ "cec" , self.api.get_setting , [ self , "system" , "cec" ] ] ,
			[ "power_mode" , self.api.get_setting , [ self , "system" , "power_mode" ] ] ,
			[ "aspect_ratio" , self.api.get_setting , [ self , "system" , "aspect_ratio" ] ] ,
			[ "input_at_power_on" , self.api.get_setting , [ self , "system" , "input_at_power_on" ] ] ,
			[ "cast_name" , self.api.get_setting , [ self , "system" , "cast_name" ] ] ,
			[ "mobile_devices" , self.api.get_setting , [ self , "system" , "mobile_devices" ] ] ,
			[ "accessibility" , self.api.get_setting , [ self , "system" , "accessibility" ] ] ,
			[ "reset_and_admin" , self.api.get_setting , [ self , "system" , "reset_and_admin" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"service_check": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"system_information": result[ 1 ] ,
			"menu_language": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"local_time_settings": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"cec": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"power_mode": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"aspect_ratio": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"input_at_power_on": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"cast_name": result[ 8 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"mobile_devices": result[ 9 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"accessibility": result[ 10 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"reset_and_admin": result[ 11 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_cast_settings( self ):
		batch_list = [
			[ "device_name" , self.api.get_setting , [ self , "cast" , "device_name" ] ] ,
			[ "session_device_id" , self.api.get_setting , [ self , "cast" , "session_device_id" ] ] ,
			[ "google_cast_id" , self.api.get_setting , [ self , "cast" , "google_cast_id" ] ] ,
			[ "google_cast_cd" , self.api.get_setting , [ self , "cast" , "google_cast_cd" ] ] ,
			[ "google_privacy_policy" , self.api.get_setting , [ self , "cast" , "google_privacy_policy" ] ] ,
			[ "tos_accepted" , self.api.get_setting , [ self , "cast" , "tos_accepted" ] ] ,
			[ "soft_ap_config" , self.api.get_setting , [ self , "cast" , "soft_ap_config" ] ] ,
			[ "start_soft_ap" , self.api.get_setting , [ self , "cast" , "start_soft_ap" ] ] ,
			[ "stop_soft_ap" , self.api.get_setting , [ self , "cast" , "stop_soft_ap" ] ] ,
			[ "soft_ap_status" , self.api.get_setting , [ self , "cast" , "soft_ap_status" ] ] ,
			[ "soft_ap_mode" , self.api.get_setting , [ self , "cast" , "soft_ap_mode" ] ] ,
			[ "state" , self.api.get_setting , [ self , "cast" , "state" ] ] ,
		]
		result = _x_batch_process({
			"max_workers": 3 ,
			"batch_list": batch_list ,
		})
		return {
			"device_name": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"session_device_id": result[ 1 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"google_cast_id": result[ 2 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"google_cast_cd": result[ 3 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"google_privacy_policy": result[ 4 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"tos_accepted": result[ 5 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"soft_ap_config": result[ 6 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"start_soft_ap": result[ 7 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"stop_soft_ap": result[ 8 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"soft_ap_status": result[ 9 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"soft_ap_mode": result[ 10 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
			"state": result[ 11 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
		}

	def get_all_settings( self ):
		# Top Level Settings Categories = [ picture , audio , network , timers , channels , closed_captions , devices , system , cast ]
		return {
			"picture": self.get_picture_settings() ,
			"audio": self.get_audio_settings() ,
			"network": self.get_network_settings() ,
			"timers": self.get_timer_settings() ,
			"channels": self.get_channel_settings() ,
			"closed_captions": self.get_closed_caption_settings() ,
			"devices": self.get_device_settings() ,
			"system": self.get_system_settings() ,
			"cast": self.get_cast_settings()
		}


#!/usr/bin/env python3

# Based on https://github.com/vkorn/pyvizio

from .discover import Discover
from .api import API
from .settings import Settings
from pprint import pprint
import sys
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor
from wakeonlan import send_magic_packet

def _x_batch_process_exec_next_iter( options ):
	if len( options ) > 2:
		result = options[ 1 ]( *options[ 2 ] )
	else:
		result = options[ 1 ]()
	return result

def _x_batch_process( options ):
	batch_size = len( options[ "batch_list" ] )
	with ThreadPoolExecutor() as executor:
		# result_pool = list( tqdm( executor.map( batch_process_exec_next_iter , iter( options[ "batch_list" ] ) ) , total=batch_size ) )
		result_pool = list( executor.map( _x_batch_process_exec_next_iter , iter( options[ "batch_list" ] ) ) )
		return result_pool

class VizioController:

	def __init__( self , options={} ):
		self.options = options
		if "mac_address" not in options:
			print("you have to send the mac adress of the tv")
			sys.exit( 1 )
		if "ip" not in options:
			self.discover = Discover( options )
			self.ip = self.discover.find_tv()
			options["ip"] = self.ip
			print( "IP == " )
			print( options["ip"] )
		else:
			self.ip = options["ip"]
		if "request_token" in options:
			if "code_displayed_on_tv" in options:
				self.api = API(options)
				options["access_token"] = self.api.pairing_stage_2( self.ip , options["request_token"] , options["code_displayed_on_tv"] )
				print( "access_token == " )
				print( str( options["access_token"] ) )
				sys.exit( 1 )
		if "access_token" not in options:
			self.api = API(options)
			request_token = self.api.pairing_stage_1( self.ip )
			print( "request_token ==" )
			print( str( request_token ) )
			print( f"Ok , now rerun this and set request_token and code_displayed_on_tv" )
			sys.exit( 1 )
		if "always_wakeup" in options:
			if options[ "always_wakeup" ] == True:
				send_magic_packet( self.options[ "mac_address" ] )

		self.api = API( options )
		self.settings = Settings( options )
		# print( "Retrieving Current Settings" )
		# self.current_volume = self.api.get_volume()
		# self.audio_settings = self.api.get_audio_settings()
		# self.audio_settings_options = self.api.get_all_audio_settings_options()
		# self.current_input = self.api.get_current_input()
		# self.available_inputs = self.api.get_available_inputs()
		# self.current_app = self.api.get_current_app()
		# self.settings_types = self.api.get_settings_types()
		# self.settings = {}
		# for index , setting in enumerate( self.settings_types["ITEMS"] ):
		# 	options = self.api.get_all_settings_for_type( setting["CNAME"] )
		# 	options = [ x["CNAME"] for x in options["ITEMS"] ]
		# 	self.settings[setting["CNAME"]] = {}
		# 	for option_index , option in enumerate( options ):
		# 		self.settings[setting["CNAME"]][ option ] = self.api.get_setting( setting["CNAME"] , option )
		# pprint( self.settings )
		# print( f"IP == {self.ip}" )

	def get_status( self ):
		try:
			batch_list = [
				[ "power_state" , self.api.get_power_state ] ,
				[ "volume" , self.api.get_volume ] ,
				[ "input" , self.api.get_current_input ] ,
				[ "app" , self.api.get_current_app ] ,
			]
			result = _x_batch_process({
				"max_workers": 3 ,
				"batch_list": batch_list ,
			})
			return {
				"power": result[ 0 ][ "ITEMS" ][ 0 ][ "VALUE" ] ,
				"volume": result[ 1 ] ,
				"input": result[ 2 ][ "name" ] ,
				"app": result[ 3 ][ "ITEM" ][ "VALUE" ][ "APP_ID" ] ,
			}
		except Exception as e:
			print( e )
			return "API SERVER OFFLINE"

	def set_volume( self , target_level ):
		current_volume = tv.api.get_volume()
		volume_difference = ( current_volume - target_level )
		volume_difference_absolute = abs( volume_difference )
		print( f"Current Volume === {current_volume}" )
		print( f"Difference to Target [{default_volume}] === {volume_difference_absolute}" )
		if current_volume > default_volume:
			for i in range( 0 , volume_difference ):
				self.api.volume_down()
				print( f"Current Volume === {self.api.get_volume()}" )
		elif current_volume < default_volume:
			for i in range( 0 , volume_difference_absolute ):
				self.api.volume_up()
				print( f"Current Volume === {self.api.get_volume()}" )



import sys
import os
import subprocess
import socket
import netifaces
import platform

class Discover:

	def __init__( self , options={} ):
		self.options = options
		self.map = {}

	def find_tv( self ):
		self.platform = platform.system()
		self.get_interfaces()
		self.get_gateways()
		self.nmap_all_gateways()
		self.arp_all_interfaces()
		return self.get_ip_from_mac_address( self.options["mac_address"] )

	def get_interfaces( self ):
		interfaces = netifaces.interfaces()
		print( interfaces )
		self.map[ "interfaces" ] = {}
		for index , interface in enumerate( interfaces ):
			self.map[ "interfaces" ][ str( interface ) ] = {}

	def get_gateways( self ):
		gateways = netifaces.gateways()
		print( gateways )
		for index , gateway in enumerate( gateways ):
			for index_item , item in enumerate( gateways[ gateway ] ):
				if isinstance( item , tuple ):
					if item[ 1 ] in self.map[ "interfaces" ]:
						self.map[ "interfaces" ][ str( item[ 1 ] ) ][ "gateway_ip" ] = item[ 0 ]
						self.map[ "interfaces" ][ str( item[ 1 ] ) ][ "ips" ] = {}

	def nmap_all_gateways( self ):
		# AKA a Network "Probe"
		shell_command = [ "nmap" ]
		if self.platform == "Linux":
			shell_command.append( "-sn" )
		elif self.platform == "Darwin":
			shell_command.append( "-sP" )
		if self.platform == "Windows":
			sys.exit( 1 )
		for index , interface in enumerate( self.map[ "interfaces"] ):
			if "gateway_ip" in self.map[ "interfaces" ][ interface ]:
				#print( "Maping " + interface )
				shell_command.append( self.map[ "interfaces" ][ interface ][ "gateway_ip" ] + "/24" )
				#print( result.returncode, result.stdout, result.stderr )
				result = subprocess.run( shell_command , capture_output=True , universal_newlines=True )

	def arp_all_interfaces( self ):
		if self.platform == "Windows":
			sys.exit( 1 )
		for index , interface in enumerate( self.map[ "interfaces"] ):
			if "gateway_ip" in self.map[ "interfaces" ][ interface ]:
				print( f'Searching: {interface} : {self.map[ "interfaces" ][ interface ][ "gateway_ip" ]}' )
				shell_command = [ "arp" , "-na" , "-i" , interface ]
				result = subprocess.run( shell_command , capture_output=True , universal_newlines=True )
				lines = result.stdout.split( "\n" )
				for index , line in enumerate( lines ):
					#print( str( index ) + " === " + line )
					if "incomplete" in line:
						continue
					if len( line ) < 3:
						continue
					mac_address = line.split( "at " )
					if len( mac_address ) < 1:
						continue
					mac_address = mac_address[ 1 ].split( " on" )
					if ( len( mac_address ) < 1 ):
						continue
					line_interface = mac_address[ 1 ].strip()
					mac_address = mac_address[ 0 ].strip()
					mac_address = mac_address.split( " " )[ 0 ]
					line_interface = line_interface.split( " ifscope" )
					if len( line_interface ) < 1:
						continue
					line_interface = line_interface[ 0 ]
					ip = line[ line.find( "(" ) + 1 : line.find( ")" ) ]
					self.map[ "interfaces" ][ line_interface ][ "ips" ][ ip ] = { "mac_address": mac_address }

	def get_ip_from_mac_address( self , mac_address ):
		for index , interface in enumerate( self.map[ "interfaces"] ):
			if "ips" in self.map[ "interfaces" ][ interface ]:
				for index_ip , ip in enumerate( self.map[ "interfaces" ][ interface ][ "ips" ] ):
					if "mac_address" in self.map[ "interfaces" ][ interface ][ "ips" ][ ip ]:
						if mac_address == self.map[ "interfaces" ][ interface ][ "ips" ][ ip ][ "mac_address" ]:
							return ip


import setuptools
import pathlib
import pkg_resources

# pipreqs ./viziocontroller

# https://stackoverflow.com/a/59971469
with pathlib.Path('./viziocontroller/requirements.txt').open() as requirements_txt:
	install_requires = [
		str(requirement)
		for requirement
		in pkg_resources.parse_requirements(requirements_txt)
	]

setuptools.setup(
	name="viziocontroller",
	version="0.1.3",
	author="7435171",
	author_email="48723247842@protonmail.com",
	description="Vizio Controller",
	url="https://github.com/48723247842/VizioController",
	packages=setuptools.find_packages(include=['viziocontroller', 'viziocontroller.*']) ,
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.8',
	install_requires=install_requires
)